@@ -48,6 +48,8 @@ static AtkObject*   find_child_in_list                (GList          *list,
                                                        gint           index);
 static void         check_cache                       (GailNotebook   *gail_notebook,
                                                        GtkNotebook    *notebook);
+static void         reset_cache                       (GailNotebook   *gail_notebook,
+                                                       gint           index);
 static void         create_notebook_page_accessible   (GailNotebook   *gail_notebook,
                                                        GtkNotebook    *notebook,
                                                        gint           index,
@@ -488,6 +490,19 @@ check_cache (GailNotebook *gail_notebook,
   gail_notebook->page_count = i;
 }
 
+static void
+reset_cache (GailNotebook *gail_notebook,
+             gint         index)
+{
+  GList *l;
+
+  for (l = gail_notebook->page_cache; l; l = l->next)
+    {
+      if (GAIL_NOTEBOOK_PAGE (l->data)->index > index)
+        GAIL_NOTEBOOK_PAGE (l->data)->index -= 1;
+    }
+}
+
 static void
 create_notebook_page_accessible (GailNotebook *gail_notebook,
                                  GtkNotebook  *notebook,
@@ -539,6 +554,7 @@ gail_notebook_real_remove_gtk (GtkContainer *container,
   g_return_val_if_fail (obj, 1);
   gail_notebook->page_cache = g_list_remove (gail_notebook->page_cache, obj);
   gail_notebook->page_count -= 1;
+  reset_cache (gail_notebook, index);
   g_signal_emit_by_name (gail_notebook,
                          "children_changed::remove",
                           GAIL_NOTEBOOK_PAGE (obj)->index, 
@@ -577,6 +593,8 @@ gail_notebook_check_focus_tab (gpointer data)
   GailNotebook *gail_notebook;
   GtkNotebook *gtk_notebook;
 
+  GDK_THREADS_ENTER ();
+
   atk_obj = ATK_OBJECT (data);
   gail_notebook = GAIL_NOTEBOOK (atk_obj);
   widget = GTK_ACCESSIBLE (atk_obj)->widget;
@@ -586,7 +604,10 @@ gail_notebook_check_focus_tab (gpointer data)
   gail_notebook->idle_focus_id = 0;
 
   if (!gtk_notebook->focus_tab)
-    return FALSE;
+    {
+      GDK_THREADS_LEAVE ();
+      return FALSE;
+    }
 
   old_focus_page_num = gail_notebook->focus_tab_page;
   focus_page_num = g_list_index (gtk_notebook->children, gtk_notebook->focus_tab->data);
@@ -599,6 +620,9 @@ gail_notebook_check_focus_tab (gpointer data)
       atk_focus_tracker_notify (obj);
       g_object_unref (obj);
     }
+
+  GDK_THREADS_LEAVE ();
+
   return FALSE;
 }
 
