@@ -183,17 +183,22 @@ notify_child_added (gpointer data)
   GailNotebookPage *page;
   AtkObject *atk_object, *atk_parent;
 
+  GDK_THREADS_ENTER ();
+
   g_return_val_if_fail (GAIL_IS_NOTEBOOK_PAGE (data), FALSE);
   page = GAIL_NOTEBOOK_PAGE (data);
   atk_object = ATK_OBJECT (data);
 
   /* The widget page->notebook may be deleted before this handler is called */
-  if (page->notebook == NULL)
-    return FALSE;
-  atk_parent = gtk_widget_get_accessible (GTK_WIDGET (page->notebook));
-  atk_object_set_parent (atk_object, atk_parent);
-  g_signal_emit_by_name (atk_parent, "children_changed::add", page->index, atk_object, NULL);
+  if (page->notebook != NULL)
+    {
+      atk_parent = gtk_widget_get_accessible (GTK_WIDGET (page->notebook));
+      atk_object_set_parent (atk_object, atk_parent);
+      g_signal_emit_by_name (atk_parent, "children_changed::add", page->index, atk_object, NULL);
+    }
   
+  GDK_THREADS_LEAVE ();
+
   return FALSE;
 }
 
